'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = getHmrString;

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _babelCore = require('babel-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getRenderArgs(args) {
  return args.map(function (arg) {
    var _generate = (0, _babelGenerator2.default)(arg),
        code = _generate.code;

    var ret = (0, _babelCore.transform)(code, {
      ast: false,
      code: true,
      babelrc: false,
      presets: ['es2015', 'react', 'stage-0'],
      plugins: [['transform-strict-mode', { strict: false }]]
    });

    return ret.code.replace(/;$/g, '').replace(/^(['"])use strict\1;/g, '').trim();
  });
}

function getComponentHmr(app, paths, args) {
  var vars = paths.map(function (_ref) {
    var name = _ref.name,
        path = _ref.path;
    return 'const ' + name + ' = require(\'' + path + '\')';
  }).join('\n    ');

  var _getRenderArgs = getRenderArgs(args),
      _getRenderArgs2 = _slicedToArray(_getRenderArgs, 3),
      component = _getRenderArgs2[0],
      container = _getRenderArgs2[1],
      callback = _getRenderArgs2[2];

  var renderFn = callback ? app + '.render(' + component + ', ' + container + ', ' + callback + ')' : app + '.render(' + component + ', ' + container + ')';

  return '\n  let render = () => {\n    ' + vars + '\n    ' + renderFn + '\n  }\n\n  // hmr\n  if (module.hot) {\n    const renderNormally = render\n    const renderException = (error) => {\n      const RedBox = require(\'redbox-react\')\n      const ReactDOM = require(\'react-dom\')\n      ReactDOM.render(React.createElement(RedBox, { error: error }), ' + container + ')\n    }\n\n    render = () => {\n      try {\n        renderNormally()\n      } catch (error) {\n        console.error(\'error\', error) // eslint-disable-line no-console\n        renderException(error)\n      }\n    }\n\n    module.hot.accept([' + paths.map(function (_ref2) {
    var path = _ref2.path;
    return '\'' + path + '\'';
  }).join(' ,') + '], () => {\n      render()\n    })\n  }\n\n  render()\n';
}

function getModelHmr(app, modelPaths) {
  return modelPaths.map(function (path) {
    return '\n    if (module.hot) {\n      const modelNamespaceMap = {};\n      modelNamespaceMap[\'' + path + '\'] = require(\'' + path + '\').namespace;\n      module.hot.accept(\'' + path + '\', () => {\n        try {\n          ' + app + '.unmodel(modelNamespaceMap[\'' + path + '\']);\n          ' + app + '.model(require(\'' + path + '\'));\n        } catch(e) { console.error(e); }\n      });\n    }\n  ';
  }).join('\n');
}

// eslint-disable-next-line
function getHmrString(app) {
  var components = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var models = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var args = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var enableModel = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

  return '\n    // Generated by babel-plugin-mickey-hmr\n    (function() {\n      console.log(\'[HMR] inited with babel-plugin-mickey-hmr\')\n      ' + getComponentHmr(app, components, args) + '\n      ' + (enableModel ? getModelHmr(app, models) : '') + '\n    })()\n  ';
}
module.exports = exports['default'];