import generate from 'babel-generator'
import { transform } from 'babel-core'


function getRenderArgs(args) {
  return args.map((arg) => {
    const { code } = generate(arg)
    const ret = transform(code, {
      ast: false,
      code: true,
      babelrc: false,
      presets: [
        'es2015',
        'react',
        'stage-0',
      ],
      plugins: [
        ['transform-strict-mode', { strict: false }],
      ],
    })

    return ret.code
      .replace(/;$/g, '')
      .replace(/^(['"])use strict\1;/g, '')
      .trim()
  })
}

function getComponentHmr(app, paths, args) {
  const vars = paths.map(({ name, path }) => (`const ${name} = require('${path}')`)).join('\n    ')
  const [component, container, callback] = getRenderArgs(args)
  const renderFn = callback
    ? `${app}.render(${component}, ${container}, ${callback})`
    : `${app}.render(${component}, ${container})`

  return `
  let render = () => {
    ${vars}
    ${renderFn}
  }

  // hmr
  if (module.hot) {
    const renderNormally = render
    const renderException = (error) => {
      const RedBox = require('redbox-react')
      const ReactDOM = require('react-dom')
      ReactDOM.render(React.createElement(RedBox, { error: error }), ${container})
    }

    render = () => {
      try {
        renderNormally()
      } catch (error) {
        console.error('error', error) // eslint-disable-line no-console
        renderException(error)
      }
    }

    module.hot.accept([${paths.map(({ path }) => `'${path}'`).join(' ,')}], () => {
      render()
    })
  }

  render()
`
}

function getModelHmr(app, modelPaths) {
  return modelPaths.map(path => `
    if (module.hot) {
      const modelNamespaceMap = {};
      modelNamespaceMap['${path}'] = require('${path}').namespace;
      module.hot.accept('${path}', () => {
        try {
          ${app}.unmodel(modelNamespaceMap['${path}']);
          ${app}.model(require('${path}'));
        } catch(e) { console.error(e); }
      });
    }
  `).join('\n')
}

// eslint-disable-next-line
export default function getHmrString(app, components = [], models = [], args = [], enableModel = false) {
  return `
    // Generated by babel-plugin-mickey-hmr
    (function() {
      console.log('[HMR] inited with babel-plugin-mickey-hmr')
      ${getComponentHmr(app, components, args)}
      ${enableModel ? getModelHmr(app, models) : ''}
    })()
  `
}
